#!/bin/bash
#Drive the bliso board in some very specific ways. (Rev 1 Model B Pi)
#This script copyright by Richard Neill (www.richardneill.org), 2013.  Free Software, licensed under the GNU GPL, version 3+.

#Note that using /usr/local/bin/gpio repeatedly is relatively slow (taking about 10ms per edge, mostly because of the new-process overhead).
#It would be much faster to write directly to /sys/class/gpio/... (using echo, which is a builtin). Eg:
#    time for i in `seq 1 500`; do gpio -g write 4 0; gpio -g write 4 1; done
#        => 10.9s => 10.9ms per edge  (mostly process overhead)
#
#    gpio export 4 out
#    time for i in `seq 1 500`; do echo 1 > /sys/class/gpio/gpio4/value; echo 0 > /sys/class/gpio/gpio4/value; done
#        => 0.41s => 0.41ms per gpio edge.  (note that echo is a bash builtin)
#
#    gpio export 4 out
#    time for i in `seq 1 500`; do /bin/echo 1 > /sys/class/gpio/gpio4/value; /bin/echo 0 > /sys/class/gpio/gpio4/value; done
#        => 9.3s => 9.3ms per gpio edge.  (/bin/echo is much slower)
#
#But for now, use the side-effect of this to get a ramping-up visual effect. TODO: refactor this.

GPIO=/usr/local/bin/gpio	#apache and cron don't necessarily have this in $PATH.
SHUTDOWN=/sbin/shutdown

#GPIO definitions.
LEDS="0 1 4 7 8 9 10 11"	#left to right
SDEL="11 10 9 8 7 4 1 0"	#right to left
EDLS="7 8 4 9 1 10 0 11"	#center outwards
LSED="0 11 1 10 4 9 7 8"	#edges inwards
BUZ=21				#buzzer
PWM=18				#pwm led
SW_BL=23			#bottom left switch
SW_BR=25			#bottom right switch

if [ $# -ne 1 -o "$1" == -h -o "$1" == --help ];then
	echo "Bliso control. One argument:"
	echo "    init         init (enable outputs, all low, required at least once)"
	echo "    off          all LEDs off"
	echo "    on           all LEDs on"
	echo "    blink        all LEDs blink"
	echo "    flash        all LEDs flash twice"
	echo "    ramp         all LEDs ramp up and down"
	echo "    silent       silence the beeper"
	echo "    offquiet     all LEDs off and silence the beeper"
	echo "    pip          very short beep"
	echo "    bip          short beep"
	echo "    beep         beep"
	echo "    Beep         long beep"
	echo "    BEEP         very long beep"
	echo "    3bips        3 short beeps"
	echo "    5min         5 minute countdown"
	echo "    alert        ramp LEDs + 3 beeps"
	echo "    remind       ramp LEDs + 3 short bips"
	echo "    nag          flash LEDs and long BEEP"
	echo "    NOW          flash LEDs and several long BEEP/bip/bips"
	echo "    pwm0         PWM led off"
	echo "    pwm1         PWM led very dim"
	echo "    pwm2         PWM led dim"
	echo "    pwm3         PWM led mid"
	echo "    pwm4         PWM led bright"
	echo "    pwm5         PWM led full"
	echo "    shutdownwait Wait for press of bottom-left button and shutdown"
	echo "    rebootwait   Wait for press of bottom-right button and reboot"
	echo "    init2        init, pip, leds, then shutdownwait/rebootwait"
	exit 1
fi

trap ctrl_c INT
function ctrl_c() {
	echo "Cancelled: silencing"
        $GPIO -g mode $BUZ out
	exit 0;
}

if [ "$1" == init ];then
	echo "Initialise"
	$GPIO reset; for X in $LEDS; do $GPIO -g mode $X output; $GPIO -g write $X 0; done ; $GPIO -g mode $PWM pwm; $GPIO -g pwm $PWM 0; $GPIO -g mode $BUZ out;

elif [ "$1" == "shutdownwait" ] ;then	#This sets up an edge-triggered non-busy-wait interrupt.  (must have initialised before now, and not again after)
	echo "Waiting for bottom-left switch; will then shutdown."	#NB subsequently re-running init will cause this process to never exit.
	$GPIO -g wfi $SW_BL falling;	#waits
	echo "shutting down now..."
	#for ((i=1023;i>0;i>>=1)); do $GPIO -g pwm $PWM $i; sleep 0.2; done
	for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.2
	for X in $LSED; do $GPIO -g write $X 0; sleep 0.1; done;
	sudo $SHUTDOWN -h now

elif [ "$1" == "rebootwait" ] ;then	#This sets up an edge-triggered non-busy-wait interrupt.  (must have initialised before now, and not again after)
	echo "Waiting for bottom-right switch; will then reboot."	#NB subsequently re-running init will cause this process to never exit.
	$GPIO -g wfi $SW_BR falling;	#waits
	echo "rebooting now..."
	#for ((i=1023;i>0;i>>=1)); do $GPIO -g pwm $PWM $i; sleep 0.1; done
	#for ((i=1;i<1023;i<<=1)); do $GPIO -g pwm $PWM $i; sleep 0.1; done
	#for ((i=1023;i>0;i>>=1)); do $GPIO -g pwm $PWM $i; sleep 0.1; done
	for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.2
	for X in $LSED; do $GPIO -g write $X 0; sleep 0.05; done;
	for X in $EDLS; do $GPIO -g write $X 1; sleep 0.05; done;
	for X in $LSED; do $GPIO -g write $X 0; sleep 0.05; done;
	sudo $SHUTDOWN -r now

elif [ "$1" == init2 ];then		#duplicate code.
	echo "Initialise and Shutdownwait"
	$0 init
	for X in $EDLS; do $GPIO -g write $X 1; done; sleep 0.15
	for X in $EDLS; do $GPIO -g write $X 0; done;
	$0 pip
	$0 shutdownwait &
	$0 rebootwait

elif [ "$1" == off ];then
	echo "All off"
	for X in $LEDS; do $GPIO -g write $X 0; done
	$GPIO -g pwm $PWM 0

elif [ "$1" == on ];then
	echo "All on"
	for X in $LEDS; do $GPIO -g write $X 1; done

elif [ "$1" == blink ];then
	echo "All blink"
	for X in $EDLS; do $GPIO -g write $X 1; done; sleep 0.1
	for X in $EDLS; do $GPIO -g write $X 0; done; 

elif [ "$1" == flash ];then
	echo "All flash"
	for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.25
	for X in $LEDS; do $GPIO -g write $X 0; done; sleep 0.25
	for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.25
	for X in $LEDS; do $GPIO -g write $X 0; done; sleep 0.25

elif [ "$1" == ramp ];then
	echo "All ramp"
	for X in $LEDS; do $GPIO -g write $X 1; sleep 0.05; done
	for X in $SDEL; do $GPIO -g write $X 0; sleep 0.05; done

elif [ "$1" == silent ];then
	echo "Silence"
	$GPIO -g mode $BUZ out

elif [ "$1" == offquiet ];then
	echo "All off, silent"
	for X in $LEDS; do $GPIO -g write $X 0; done
	$GPIO -g mode $BUZ out

elif [ "$1" == pip ];then
	echo "Tiny pip"
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; $GPIO -g mode $BUZ out

elif [ "$1" == bip ];then
	echo "Short bip"
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.02; $GPIO -g mode $BUZ out

elif [ "$1" == beep ];then
	echo "Beep"
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.1; $GPIO -g mode $BUZ out

elif [ "$1" == Beep ];then
	echo "long beep"
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.5; $GPIO -g mode $BUZ out

elif [ "$1" == BEEP ];then
	echo "very long beep"
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 1.5; $GPIO -g mode $BUZ out

elif [ "$1" == 3bips ];then
	echo "3 short beeps"
	for ((i=0;i<3;i++)) ; do $GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.02; $GPIO -g mode $BUZ out; sleep 0.1; done

elif [ "$1" == 5min ];then
	echo "One beep and 5 minute countdown, then beep again." 
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.01; $GPIO -g mode $BUZ out;
	#NB this process is long-lived. A simple way to lock against itself is to look at the state of the first LED.
	list=($LEDS); first=${list[0]}; state=$($GPIO -g read $first)
	if [ "$state" == 1 ];then
		echo "already running."
	else
	        for X in $LEDS; do $GPIO -g write $X 1; done
		for X in $SDEL; do sleep 37; $GPIO -g write $X 0; done; 	#8 LEDs , 37 sec = 5 min.
		$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.02; $GPIO -g mode $BUZ out;
	fi

elif [ "$1" == alert ];then
	echo "ramp LEDs + 3 beeps"
        for ((i=0;i<3;i++)) ; do
		for X in $LEDS; do $GPIO -g write $X 1; sleep 0.05; done
	       	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.02; $GPIO -g mode $BUZ out; sleep 0.1;
	        for X in $SDEL; do $GPIO -g write $X 0; sleep 0.05; done
	done
	for ((i=0;i<5;i++)) ; do
	        for X in $LEDS; do $GPIO -g write $X 1; sleep 0.05; done ; sleep 0.1
		for X in $SDEL; do $GPIO -g write $X 0; sleep 0.05; done;  sleep 0.1
	done

elif [ "$1" == remind ];then
	echo "ramp LEDs x5 + 3 short beeps"
	for X in $LEDS; do $GPIO -g write $X 1; sleep 0.05; done
        for ((i=0;i<3;i++)) ; do $GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000; sleep 0.02; $GPIO -g mode $BUZ out; sleep 0.1; done
        for X in $SDEL; do $GPIO -g write $X 0; sleep 0.05; done
	for ((i=0;i<5;i++)) ; do
	        for X in $LEDS; do $GPIO -g write $X 1; sleep 0.03; done ; sleep 0.1
		for X in $SDEL; do $GPIO -g write $X 0; sleep 0.03; done;  sleep 0.1
	done

elif [ "$1" == nag ];then
	echo "flash LEDs + long BEEP"
	$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000;
	for ((i=0;i<1;i++)) ; do
	        for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.2
	        for X in $LEDS; do $GPIO -g write $X 0; done; sleep 0.2
	done
	$GPIO -g mode $BUZ out
	for ((i=0;i<9;i++)) ; do
	        for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.2
	        for X in $LEDS; do $GPIO -g write $X 0; done; sleep 0.2
	done

elif [ "$1" == NOW ];then
	echo "flash LEDs + several long BEEP/bip/bips"
	for ((j=0;j<3;j++)) ; do
	for ((i=0;i<20;i++)) ; do
	        for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.05
		if [ $i == 0 -o $i == 3 -o $i == 5 -o $i == 7 ];then 	
			$GPIO -g mode $BUZ clock; $GPIO -g clock $BUZ 4000;
		fi
	        for X in $LEDS; do $GPIO -g write $X 0; done; sleep 0.05
		if [ $i == 1 -o $i == 3 -o $i == 5 -o $i == 7 ];then 	
			$GPIO -g mode $BUZ out
		fi
	done
	done
	for ((i=0;i<20;i++)) ; do
	        for X in $LEDS; do $GPIO -g write $X 1; done; sleep 0.05
	        for X in $LEDS; do $GPIO -g write $X 0; done; sleep 0.05
	done

elif [ "$1" == pwm0 ];then
	echo "PWM led off"
	$GPIO -g pwm $PWM 0

elif [ "$1" == pwm1 ];then
	echo "PWM led very dim"
	$GPIO -g pwm $PWM 32

elif [ "$1" == pwm2 ];then
	echo "PWM led dim"
	$GPIO -g pwm $PWM 128

elif [ "$1" == pwm3 ];then
	echo "PWM led mid"
	$GPIO -g pwm $PWM 256

elif [ "$1" == pwm4 ];then
	echo "PWM led bright"
	$GPIO -g pwm $PWM 512

elif [ "$1" == pwm5 ];then
	echo "PWM led full"
	$GPIO -g pwm $PWM 1023


else
	echo "Unrecognised command. Use -h for help"
	exit 1
fi

exit 0


